@@ -1,24 +1,7 @@
-(* Lightweight thread library for OCaml
- * http://www.ocsigen.org/lwt
- * Interface Lwt_sequence
- * Copyright (C) 2009 Jérémie Dimino
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU Lesser General Public License as
- * published by the Free Software Foundation, with linking exceptions;
- * either version 2.1 of the License, or (at your option) any later
- * version. See COPYING file for details.
- *
- * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
- *)
+(* This file is part of Lwt, released under the MIT license. See LICENSE.md for
+   details, or visit https://github.com/ocsigen/lwt/blob/master/LICENSE.md. *)
+
+
 
 (** Mutable sequence of elements (deprecated) *)
 
@@ -31,10 +14,14 @@
     - removing an element while the sequence is being transversed.
 
    @deprecated This module should be an internal implementation detail of Lwt,
-   and may be removed from the API at some point in the future. Use any other
-   doubly-linked list library as an alternative.
+   and may be removed from the API at some point in the future. Use package
+   {{:https://github.com/mirage/lwt-dllist} [lwt-dllist]} instead.
 *)
 
+[@@@ocaml.deprecated
+" Use package lwt-dllist. See
+   https://github.com/mirage/lwt-dllist"]
+
 type 'a t
   (** Type of a sequence holding values of type ['a] *)
 
@@ -62,7 +49,7 @@ val is_empty : 'a t -> bool
   (** Returns [true] iff the given sequence is empty *)
 
 val length : 'a t -> int
-  (** Returns the number of elemenets in the given sequence. This is a
+  (** Returns the number of elements in the given sequence. This is a
       O(n) operation where [n] is the number of elements in the
       sequence. *)
 
