@@ -1,30 +1,10 @@
-(* Lightweight thread library for OCaml
- * http://www.ocsigen.org/lwt
- * Module Lwt_unix
- * Copyright (C) 2010 Jérémie Dimino
- *               2010 Pierre Chambart
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU Lesser General Public License as
- * published by the Free Software Foundation, with linking exceptions;
- * either version 2.1 of the License, or (at your option) any later
- * version. See COPYING file for details.
- *
- * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
- *)
+(* This file is part of Lwt, released under the MIT license. See LICENSE.md for
+   details, or visit https://github.com/ocsigen/lwt/blob/master/LICENSE.md. *)
+
+
 
 open Bigarray
 
-open Lwt.Infix
-
 type t = (char, int8_unsigned_elt, c_layout) Array1.t
 
 let create size = Array1.create char c_layout size
@@ -60,7 +40,7 @@ let blit_from_bytes src_buf src_ofs dst_buf dst_ofs len =
   if (len < 0
       || src_ofs < 0 || src_ofs > Bytes.length src_buf - len
       || dst_ofs < 0 || dst_ofs > length dst_buf - len) then
-    invalid_arg "String.blit"
+    invalid_arg "Lwt_bytes.blit_from_bytes"
   else
     unsafe_blit_from_bytes src_buf src_ofs dst_buf dst_ofs len
 
@@ -68,7 +48,7 @@ let blit_to_bytes src_buf src_ofs dst_buf dst_ofs len =
   if (len < 0
       || src_ofs < 0 || src_ofs > length src_buf - len
       || dst_ofs < 0 || dst_ofs > Bytes.length dst_buf - len) then
-    invalid_arg "String.blit"
+    invalid_arg "Lwt_bytes.blit_to_bytes"
   else
     unsafe_blit_to_bytes src_buf src_ofs dst_buf dst_ofs len
 
@@ -76,7 +56,7 @@ let blit src_buf src_ofs dst_buf dst_ofs len =
   if (len < 0
       || src_ofs < 0 || src_ofs > length src_buf - len
       || dst_ofs < 0 || dst_ofs > length dst_buf - len) then
-    invalid_arg "String.blit"
+    invalid_arg "Lwt_bytes.blit"
   else
     unsafe_blit src_buf src_ofs dst_buf dst_ofs len
 
@@ -119,39 +99,17 @@ let copy buf =
 
 open Lwt_unix
 
-external stub_read : Unix.file_descr -> t -> int -> int -> int = "lwt_unix_bytes_read"
-external read_job : Unix.file_descr -> t -> int -> int -> int job = "lwt_unix_bytes_read_job"
+let read =
+  Lwt_unix.read_bigarray "Lwt_bytes.read" [@ocaml.warning "-3"]
 
-let read fd buf pos len =
-  if pos < 0 || len < 0 || pos > length buf - len then
-    invalid_arg "Lwt_bytes.read"
-  else
-    blocking fd >>= function
-      | true ->
-          wait_read fd >>= fun () ->
-          run_job (read_job (unix_file_descr fd) buf pos len)
-      | false ->
-          wrap_syscall Read fd (fun () -> stub_read (unix_file_descr fd) buf pos len)
-
-external stub_write : Unix.file_descr -> t -> int -> int -> int = "lwt_unix_bytes_write"
-external write_job : Unix.file_descr -> t -> int -> int -> int job = "lwt_unix_bytes_write_job"
-
-let write fd buf pos len =
-  if pos < 0 || len < 0 || pos > length buf - len then
-    invalid_arg "Lwt_bytes.write"
-  else
-    blocking fd >>= function
-      | true ->
-          wait_write fd >>= fun () ->
-          run_job (write_job (unix_file_descr fd) buf pos len)
-      | false ->
-          wrap_syscall Write fd (fun () -> stub_write (unix_file_descr fd) buf pos len)
+let write =
+  Lwt_unix.write_bigarray "Lwt_bytes.write" [@ocaml.warning "-3"]
 
 external stub_recv : Unix.file_descr -> t -> int -> int -> Unix.msg_flag list -> int = "lwt_unix_bytes_recv"
 
 let recv fd buf pos len flags =
   if pos < 0 || len < 0 || pos > length buf - len then
-    invalid_arg "recv"
+    invalid_arg "Lwt_bytes.recv"
   else
     wrap_syscall Read fd (fun () -> stub_recv (unix_file_descr fd) buf pos len flags)
 
@@ -159,7 +117,7 @@ external stub_send : Unix.file_descr -> t -> int -> int -> Unix.msg_flag list ->
 
 let send fd buf pos len flags =
   if pos < 0 || len < 0 || pos > length buf - len then
-    invalid_arg "send"
+    invalid_arg "Lwt_bytes.send"
   else
     wrap_syscall Write fd (fun () -> stub_send (unix_file_descr fd) buf pos len flags)
 
@@ -175,32 +133,18 @@ let io_vector ~buffer ~offset ~length = ({
   iov_length = length;
 } : io_vector)
 
-let check_io_vectors func_name iovs =
-  List.iter
-    (fun (iov : io_vector) ->
-       if iov.iov_offset < 0
-         || iov.iov_length < 0
-         || iov.iov_offset > length iov.iov_buffer - iov.iov_length then
-           invalid_arg func_name)
-    iovs
-
-external stub_recv_msg : Unix.file_descr -> int -> io_vector list -> int * Unix.file_descr list = "lwt_unix_bytes_recv_msg"
+let convert_io_vectors old_io_vectors =
+  let io_vectors = IO_vectors.create () in
+  old_io_vectors
+  |> List.iter (fun ({iov_buffer; iov_offset; iov_length} : io_vector) ->
+    IO_vectors.append_bigarray io_vectors iov_buffer iov_offset iov_length);
+  io_vectors
 
 let recv_msg ~socket ~io_vectors =
-  check_io_vectors "recv_msg" io_vectors;
-  let n_iovs = List.length io_vectors in
-  wrap_syscall Read socket
-    (fun () ->
-       stub_recv_msg (unix_file_descr socket) n_iovs io_vectors)
-
-external stub_send_msg : Unix.file_descr -> int -> io_vector list -> int -> Unix.file_descr list -> int = "lwt_unix_bytes_send_msg"
+  Lwt_unix.recv_msg ~socket ~io_vectors:(convert_io_vectors io_vectors)
 
 let send_msg ~socket ~io_vectors ~fds =
-  check_io_vectors "send_msg" io_vectors;
-  let n_iovs = List.length io_vectors and n_fds = List.length fds in
-  wrap_syscall Write socket
-    (fun () ->
-       stub_send_msg (unix_file_descr socket) n_iovs io_vectors n_fds fds)
+  Lwt_unix.send_msg ~socket ~io_vectors:(convert_io_vectors io_vectors) ~fds
 
 external stub_recvfrom : Unix.file_descr -> t -> int -> int -> Unix.msg_flag list -> int * Unix.sockaddr = "lwt_unix_bytes_recvfrom"
 
@@ -223,12 +167,8 @@ let sendto fd buf pos len flags addr =
    +-----------------------------------------------------------------+ *)
 
 let map_file ~fd ?pos ~shared ?(size=(-1)) () =
-  Array1.map_file fd ?pos char c_layout shared size
-    [@@ocaml.warning "-3"]
-    (* BigArray.Array1.map_file is deprecated in OCaml 4.05; however, the
-       suggested replacement requires 4.05 (Lwt still supports 4.02). The
-       replacement also has slighty different exception semantics; see
-       deprecation warning on BigArray.Array1.map_file. *)
+  Mmap.V1.map_file fd ?pos char c_layout shared [|size|]
+  |> Bigarray.array1_of_genarray
 
 [@@@ocaml.warning "-3"]
 external mapped : t -> bool = "lwt_unix_mapped" "noalloc"
@@ -240,6 +180,10 @@ type advice =
   | MADV_SEQUENTIAL
   | MADV_WILLNEED
   | MADV_DONTNEED
+  | MADV_MERGEABLE
+  | MADV_UNMERGEABLE
+  | MADV_HUGEPAGE
+  | MADV_NOHUGEPAGE
 
 external stub_madvise : t -> int -> int -> advice -> unit = "lwt_unix_madvise"
 
@@ -258,7 +202,8 @@ external stub_mincore : t -> int -> int -> bool array -> unit = "lwt_unix_mincor
 let mincore buffer offset states =
   if (offset mod page_size <> 0
       || offset < 0
-      || offset > length buffer - (Array.length states * page_size)) then
+      || length buffer - offset < (Array.length states - 1) * page_size + 1)
+  then
     invalid_arg "Lwt_bytes.mincore"
   else
     stub_mincore buffer offset (Array.length states * page_size) states
