@@ -1,24 +1,7 @@
-(* Lightweight thread library for OCaml
- * http://www.ocsigen.org/lwt
- * Interface Lwt_unix
- * Copyright (C) 2010 Jérémie Dimino
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU Lesser General Public License as
- * published by the Free Software Foundation, with linking exceptions;
- * either version 2.1 of the License, or (at your option) any later
- * version. See COPYING file for details.
- *
- * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
- *)
+(* This file is part of Lwt, released under the MIT license. See LICENSE.md for
+   details, or visit https://github.com/ocsigen/lwt/blob/master/LICENSE.md. *)
+
+
 
 (** Byte arrays *)
 
@@ -83,10 +66,10 @@ val unsafe_blit : t -> int -> t -> int -> int -> unit
   (** Same as {!blit} but without bound checking. *)
 
 val unsafe_blit_from_bytes : bytes -> int -> t -> int -> int -> unit
-  (** Same as {!blit_string_bytes} but without bounds checking. *)
+  (** Same as {!Lwt_bytes.blit_from_bytes} but without bounds checking. *)
 
 val unsafe_blit_to_bytes : t -> int -> bytes -> int -> int -> unit
-  (** Same as {!blit_bytes_string} but without bounds checking. *)
+  (** Same as {!Lwt_bytes.blit_to_bytes} but without bounds checking. *)
 
 val proxy : t -> int -> int -> t
   (** [proxy buffer offset length] creates a ``proxy''. The returned
@@ -141,10 +124,16 @@ type io_vector = {
 val io_vector : buffer : t -> offset : int -> length : int -> io_vector
 
 val recv_msg : socket : Lwt_unix.file_descr -> io_vectors : io_vector list -> (int * Unix.file_descr list) Lwt.t
-(** Not implemented on Windows. *)
+  [@@ocaml.deprecated " Use Lwt_unix.Versioned.recv_msg_2."]
+(** Not implemented on Windows.
+
+    @deprecated Use {!Lwt_unix.Versioned.recv_msg_2}. *)
 
 val send_msg : socket : Lwt_unix.file_descr -> io_vectors : io_vector list -> fds : Unix.file_descr list -> int Lwt.t
-(** Not implemented on Windows. *)
+  [@@ocaml.deprecated " Use Lwt_unix.Versioned.send_msg_2."]
+(** Not implemented on Windows.
+
+    @deprecated Use {!Lwt_unix.Versioned.send_msg_2}. *)
 
 (** {2 Memory mapped files} *)
 
@@ -165,6 +154,10 @@ type advice =
   | MADV_SEQUENTIAL
   | MADV_WILLNEED
   | MADV_DONTNEED
+  | MADV_MERGEABLE
+  | MADV_UNMERGEABLE
+  | MADV_HUGEPAGE
+  | MADV_NOHUGEPAGE
 
 val madvise : t -> int -> int -> advice -> unit
   (** [madvise buffer pos len advice] advises the kernel how the
