@@ -1,24 +1,7 @@
-(* Lightweight thread library for OCaml
- * http://www.ocsigen.org/lwt
- * Interface Lwt_engine
- * Copyright (C) 2011 Jérémie Dimino
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU Lesser General Public License as
- * published by the Free Software Foundation, with linking exceptions;
- * either version 2.1 of the License, or (at your option) any later
- * version. See COPYING file for details.
- *
- * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
- *)
+(* This file is part of Lwt, released under the MIT license. See LICENSE.md for
+   details, or visit https://github.com/ocsigen/lwt/blob/master/LICENSE.md. *)
+
+
 
 (** Lwt unix main loop engine *)
 
@@ -100,8 +83,8 @@ class virtual abstract : object
 
   (** Notes:
 
-      - the callback passed to register methods is of type [unit ->
-      unit] and not [event -> unit]
+      - the callback passed to register methods is of type [unit -> unit]
+      and not [event -> unit]
       - register methods return a lazy value which unregisters the
       event when forced
   *)
@@ -147,7 +130,7 @@ end
 
 (** Engine based on libev. If not compiled with libev support, the
     creation of the class will raise {!Lwt_sys.Not_available}. *)
-class libev : object
+class libev : ?backend:Ev_backend.t -> unit -> object
   inherit t
 
   val loop : ev_loop
@@ -156,13 +139,6 @@ class libev : object
   method loop : ev_loop
     (** Returns [loop]. *)
 end
-[@@ocaml.deprecated
-" This class will soon have parameters for selecting a libev backend. This will
- be a breaking change in Lwt 3.0.0. See
-   https://github.com/ocsigen/lwt/pull/269
- To preserve the current signature, use Lwt_engine.Versioned.libev_1
- To use the replacement immediately, use Lwt_engine.Versioned.libev_2 ()
- Both alternatives require Lwt >= 2.7.0."]
 
 (** Engine based on [Unix.select]. *)
 class select : t
@@ -223,15 +199,23 @@ sig
     method loop : ev_loop
   end
   [@@ocaml.deprecated
-   "Deprecated in favor of Lwt_engine.Versioned.libev_2. See
+" Deprecated in favor of Lwt_engine.libev. See
   https://github.com/ocsigen/lwt/pull/269"]
-  (** @deprecated In favor of {!libev_2}.
+  (** Old version of {!Lwt_engine.libev}. The current {!Lwt_engine.libev} allows
+      selecting the libev back end.
+
+      @deprecated Use {!Lwt_engine.libev}.
       @since 2.7.0 *)
 
-  (** @since 2.7.0 *)
   class libev_2 : ?backend:Ev_backend.t -> unit -> object
     inherit t
     val loop : ev_loop
     method loop : ev_loop
   end
+  [@@ocaml.deprecated
+" In Lwt >= 3.0.0, this is an alias for Lwt_engine.libev."]
+  (** Since Lwt 3.0.0, this is just an alias for {!Lwt_engine.libev}.
+
+      @deprecated Use {!Lwt_engine.libev}.
+      @since 2.7.0 *)
 end
