@@ -1,24 +1,7 @@
-(* Lightweight thread library for OCaml
- * http://www.ocsigen.org/lwt
- * Interface Lwt_io
- * Copyright (C) 2009 Jérémie Dimino
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU Lesser General Public License as
- * published by the Free Software Foundation, with linking exceptions;
- * either version 2.1 of the License, or (at your option) any later
- * version. See COPYING file for details.
- *
- * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
- *)
+(* This file is part of Lwt, released under the MIT license. See LICENSE.md for
+   details, or visit https://github.com/ocsigen/lwt/blob/master/LICENSE.md. *)
+
+
 
 (** Buffered byte channels *)
 
@@ -184,7 +167,9 @@ val atomic : ('a channel -> 'b Lwt.t) -> ('a channel -> 'b Lwt.t)
       - [atomic] can be called inside another [atomic] *)
 
 val file_length : string -> int64 Lwt.t
-  (** Returns the length of a file *)
+(** Retrieves the length of the file at the given path. If the path refers to a
+    directory, the returned promise is rejected with
+    [Unix.(Unix_error (EISDIR, _, _))]. *)
 
 val buffered : 'a channel -> int
   (** [buffered oc] returns the number of bytes in the buffer *)
@@ -206,6 +191,10 @@ val is_busy : 'a channel -> bool
       busy. A channel is busy when there is at least one job using it
       that has not yet terminated. *)
 
+val is_closed : 'a channel -> bool
+  (** [is_closed channel] returns whether the given channel is currently
+      closed. *)
+
 (** {2 Random access} *)
 
 val position : 'a channel -> int64
@@ -229,7 +218,7 @@ val read_char : input_channel -> char Lwt.t
       @raise End_of_file if the end of the file is reached *)
 
 val read_char_opt : input_channel -> char option Lwt.t
-  (** Same as {!read_byte} but does not raise [End_of_file] on end of
+  (** Same as {!Lwt_io.read_char}, but does not raise [End_of_file] on end of
       input *)
 
 val read_chars : input_channel -> char Lwt_stream.t
@@ -241,7 +230,7 @@ val read_line : input_channel -> string Lwt.t
       without the end of line. End of line is either ["\n"] or
       ["\r\n"].
 
-      If the end of line is reached before reading any character,
+      If the end of input is reached before reading any character,
       [End_of_file] is raised. If it is reached before reading an end
       of line but characters have already been read, they are
       returned. *)
@@ -254,9 +243,13 @@ val read_lines : input_channel -> string Lwt_stream.t
   (** [read_lines ic] returns a stream holding all lines of [ic] *)
 
 val read : ?count : int -> input_channel -> string Lwt.t
-  (** [read ?count ic] reads at most [count] characters from [ic]. It
-      returns [""] if the end of input is reached. If [count] is not
-      specified, it reads all bytes until the end of input. *)
+(** If [~count] is specified, [read ~count ic] reads at most [~count] characters
+    from [ic]. Note that fewer than [~count] characters can be read; check the
+    size of the resulting string. [read] returns [""] if the end of input is
+    reached.
+
+    If [~count] is not specified, [read ic] reads all bytes until the end of
+    input. *)
 
 val read_into : input_channel -> bytes -> int -> int -> int Lwt.t
   (** [read_into ic buffer offset length] reads up to [length] bytes,
@@ -273,7 +266,18 @@ val read_into_exactly : input_channel -> bytes -> int -> int -> unit Lwt.t
       @raise End_of_file on end of input *)
 
 val read_value : input_channel -> 'a Lwt.t
-  (** [read_value ic] reads a marshaled value from [ic] *)
+(** [read_value channel] reads a marshaled value from [channel]; it corresponds
+    to the standard library's
+    {{:https://caml.inria.fr/pub/docs/manual-ocaml/libref/Marshal.html#VALfrom_channel} [Marshal.from_channel]}.
+    The corresponding writing function is {!write_value}.
+
+    Note that reading marshaled values is {e not}, in general, type-safe. See
+    the warning in the description of module
+    {{:https://caml.inria.fr/pub/docs/manual-ocaml/libref/Marshal.html}
+    [Marshal]} for details. The short version is: if you read a value of one
+    type, such as [string], when a value of another type, such as [int] has
+    actually been marshaled to [channel], you may get arbitrary behavior,
+    including segmentation faults, access violations, security bugs, etc. *)
 
 (** {2 Writing} *)
 
@@ -313,11 +317,19 @@ val write_from_exactly : output_channel -> bytes -> int -> int -> unit Lwt.t
   (** [write_from_exactly oc buffer offset length] writes all [length]
       bytes from [buffer] at offset [offset] to [oc] *)
 
-val write_from_string_exactly : output_channel -> string -> int -> int -> unit Lwt.t
+val write_from_string_exactly :
+  output_channel -> string -> int -> int -> unit Lwt.t
   (** See {!write_from_exactly}. *)
 
-val write_value : output_channel -> ?flags : Marshal.extern_flags list -> 'a -> unit Lwt.t
-  (** [write_value oc ?flags x] marshals the value [x] to [oc] *)
+val write_value :
+  output_channel -> ?flags : Marshal.extern_flags list -> 'a -> unit Lwt.t
+(** [write_value channel ?flags v] writes [v] to [channel] using the [Marshal]
+    module of the standard library. See
+    {{:https://caml.inria.fr/pub/docs/manual-ocaml/libref/Marshal.html#VALto_channel}
+    [Marshal.to_channel]} for an explanation of [?flags].
+
+    The corresponding reading function is {!read_value}. See warnings about type
+    safety in the description of {!read_value}. *)
 
 (** {2 Printing} *)
 
@@ -340,16 +352,34 @@ val write_value : output_channel -> ?flags : Marshal.extern_flags list -> 'a ->
 
 val fprint : output_channel -> string -> unit Lwt.t
 val fprintl : output_channel -> string -> unit Lwt.t
+
 val fprintf : output_channel -> ('a, unit, string, unit Lwt.t) format4 -> 'a
+(** [%!] does nothing here. To flush the channel, use [Lwt_io.flush channel]. *)
+
 val fprintlf : output_channel -> ('a, unit, string, unit Lwt.t) format4 -> 'a
+(** [%!] does nothing here. To flush the channel, use [Lwt_io.flush channel]. *)
+
 val print : string -> unit Lwt.t
 val printl : string -> unit Lwt.t
+
 val printf : ('a, unit, string, unit Lwt.t) format4 -> 'a
+(** [%!] does nothing here. To flush the channel, use
+    [Lwt_io.(flush stdout)]. *)
+
 val printlf : ('a, unit, string, unit Lwt.t) format4 -> 'a
+(** [%!] does nothing here. To flush the channel, use
+    [Lwt_io.(flush stdout)]. *)
+
 val eprint : string -> unit Lwt.t
 val eprintl : string -> unit Lwt.t
+
 val eprintf : ('a, unit, string, unit Lwt.t) format4 -> 'a
+(** [%!] does nothing here. To flush the channel, use
+    [Lwt_io.(flush stderr)]. *)
+
 val eprintlf : ('a, unit, string, unit Lwt.t) format4 -> 'a
+(** [%!] does nothing here. To flush the channel, use
+    [Lwt_io.(flush stderr)]. *)
 
 (** {2 Utilities} *)
 
@@ -366,27 +396,142 @@ type file_name = string
     (** Type of file names *)
 
 val open_file :
-  ?buffer : Lwt_bytes.t ->
-  ?flags : Unix.open_flag list ->
-  ?perm : Unix.file_perm ->
-  mode : 'a mode ->
-  file_name -> 'a channel Lwt.t
-  (** [open_file ?buffer ?flags ?perm ~mode filename] opens the
-      file with name [filename] and returns a channel for
-      reading/writing it.
+  ?buffer:Lwt_bytes.t ->
+  ?flags:Unix.open_flag list ->
+  ?perm:Unix.file_perm ->
+  mode:'a mode ->
+  file_name ->
+    'a channel Lwt.t
+(** [Lwt_io.open_file ~mode file] opens the given file, either for reading (with
+    [~mode:Input]) or for writing (with [~mode:Output]). The returned channel
+    provides buffered I/O on the file.
 
-      @raise Unix.Unix_error on error.
-  *)
+    If [~buffer] is supplied, it is used as the I/O buffer.
+
+    If [~flags] is supplied, the file is opened with the given flags (see
+    {{: https://caml.inria.fr/pub/docs/manual-ocaml/libref/Unix.html#TYPEopen_flag}
+    [Unix.open_flag]}). Note that [~flags] is used {e exactly} as given. For
+    example, opening a file with [~flags] and [~mode:Input] does {e not}
+    implicitly add [O_RDONLY]. So, you should include [O_RDONLY] when opening
+    for reading ([~mode:Input]), and [O_WRONLY] when opening for writing
+    ([~mode:Input]). It is also recommended to include [O_NONBLOCK], unless you
+    are sure that the file cannot be a socket or a named pipe.
+
+    The default permissions used for creating new files are [0o666], i.e.
+    reading and writing are allowed for the file owner, group, and everyone.
+    These default permissions can be overridden by supplying [~perm].
+
+    Note: if opening for writing ([~mode:Output]), and the file already exists,
+    [open_file] truncates (clears) the file by default. If you would like to
+    keep the pre-existing contents of the file, use the [~flags] parameter to
+    pass a custom flags list that does not include [Unix.O_TRUNC].
+
+    @raise Unix.Unix_error on error. *)
 
 val with_file :
-  ?buffer : Lwt_bytes.t ->
-  ?flags : Unix.open_flag list ->
-  ?perm : Unix.file_perm ->
-  mode : 'a mode ->
-  file_name -> ('a channel -> 'b Lwt.t) -> 'b Lwt.t
-  (** [with_file ?buffer ?flags ?perm ~mode filename f] opens a
-      file and passes the channel to [f]. It is ensured that the
-      channel is closed when [f ch] terminates (even if it fails). *)
+  ?buffer:Lwt_bytes.t ->
+  ?flags:Unix.open_flag list ->
+  ?perm:Unix.file_perm ->
+  mode:'a mode ->
+  file_name ->
+  ('a channel -> 'b Lwt.t) ->
+    'b Lwt.t
+(** [Lwt_io.with_file ~mode filename f] opens the given using
+    {!Lwt_io.open_file}, and passes the resulting channel to [f].
+    [Lwt_io.with_file] ensures that the channel is closed when the promise
+    returned by [f] resolves, or if [f] raises an exception.
+
+    See {!Lwt_io.open_file} for a description of the arguments, warnings, and
+    other notes. *)
+
+val open_temp_file :
+  ?buffer:Lwt_bytes.t ->
+  ?flags:Unix.open_flag list ->
+  ?perm:Unix.file_perm ->
+  ?temp_dir:string ->
+  ?prefix:string ->
+  ?suffix:string ->
+  unit ->
+    (string * output_channel) Lwt.t
+(** [open_temp_file ()] starts creating a new temporary file, and evaluates to a
+    promise for the pair of the file's name, and an output channel for writing
+    to the file.
+
+    The caller should take care to delete the file later. Alternatively, see
+    {!Lwt_io.with_temp_file}.
+
+    The [?buffer] and [?perm] arguments are passed directly to an internal call
+    to {!Lwt_io.open_file}.
+
+    If not specified, [?flags] defaults to
+    [[O_CREATE; O_EXCL; O_WRONLY; O_CLOEXEC]]. If specified, the specified flags
+    are used exactly. Note that these should typically contain at least
+    [O_CREAT] and [O_EXCL], otherwise [open_temp_file] may open an existing
+    file.
+
+    [?temp_dir] can be used to choose the directory in which the file is
+    created. For the current directory, use
+    {{: https://caml.inria.fr/pub/docs/manual-ocaml/libref/Filename.html#VALcurrent_dir_name}
+    [Filename.current_dir_name]}. If not specified, the directory is taken from
+    {{: https://caml.inria.fr/pub/docs/manual-ocaml/libref/Filename.html#VALget_temp_dir_name}
+    [Filename.get_temp_dir_name]}, which is typically set to your system
+    temporary file directory.
+
+    [?prefix] helps determine the name of the file. It will be the prefix
+    concatenated with a random sequence of characters. If not specified,
+    [open_temp_file] uses some default prefix.
+
+    [?suffix] is like prefix, but it is appended at the end of the filename. In
+    particular, it can be used to set the extension. *)
+
+val with_temp_file :
+  ?buffer:Lwt_bytes.t ->
+  ?flags:Unix.open_flag list ->
+  ?perm:Unix.file_perm ->
+  ?temp_dir:string ->
+  ?prefix:string ->
+  ?suffix:string ->
+  (string * output_channel -> 'b Lwt.t) ->
+    'b Lwt.t
+(** [with_temp_file f] calls {!open_temp_file}[ ()], passing all optional
+    arguments directly to it. It then attaches [f] to run after the file is
+    created, passing the filename and output channel to [f]. When the promise
+    returned by [f] is resolved, [with_temp_file] closes the channel and deletes
+    the temporary file by calling {!Lwt_unix.unlink}. *)
+
+val create_temp_dir :
+  ?perm:Unix.file_perm ->
+  ?parent:string ->
+  ?prefix:string ->
+  ?suffix:string ->
+  unit ->
+    string Lwt.t
+(** Creates a temporary directory, and returns a promise that resolves to its
+    path. The caller must take care to remove the directory. Alternatively, see
+    {!Lwt_io.with_temp_dir}.
+
+    If [~perm] is specified, the directory is created with the given
+    permissions. The default permissions are [0755].
+
+    [~parent] is the directory in which the temporary directory is created. If
+    not specified, the default value is the result of
+    [Filename.get_temp_dir_name ()].
+
+    [~prefix] is prepended to the directory name, and [~suffix] is appended to
+    it. *)
+
+val with_temp_dir :
+  ?perm:Unix.file_perm ->
+  ?parent:string ->
+  ?prefix:string ->
+  ?suffix:string ->
+  (string -> 'a Lwt.t) ->
+    'a Lwt.t
+(** [with_temp_dir f] first calls {!create_temp_dir}, forwarding all optional
+    arguments to it. Once the temporary directory is created at [path],
+    [with_temp_dir f] calls [f path]. When the promise returned by [f path] is
+    resolved, [with_temp_dir f] recursively deletes the temporary directory and
+    all its contents. *)
 
 val open_connection :
   ?fd : Lwt_unix.file_descr ->
@@ -396,7 +541,7 @@ val open_connection :
       connection to the given address and returns two channels for using
       it. If [fd] is not specified, a fresh one will be used.
 
-      The connection is completly closed when you close both
+      The connection is completely closed when you close both
       channels.
 
       @raise Unix.Unix_error on error.
@@ -410,66 +555,101 @@ val with_connection :
       connection to the given address and passes the channels to
       [f] *)
 
+(**/**)
+
+(** This function is not public API and can be changed or removed without
+    notice. It is exposed in order to test [with_connection].
+
+    [with_close_connection f (ic, oc)] calls [f (ic, oc)] and makes sure that
+    [ic] and [oc] are closed, whether [f] returns or fails with an exception.
+    Does not fail if [ic] or [oc] is already closed. *)
+val with_close_connection :
+  (input_channel * output_channel -> 'a Lwt.t) ->
+  input_channel * output_channel ->
+  'a Lwt.t
+
+(**/**)
+
 type server
   (** Type of a server *)
 
-val establish_server :
-  ?fd : Lwt_unix.file_descr ->
-  ?buffer_size : int ->
-  ?backlog : int ->
-  Unix.sockaddr -> (input_channel * output_channel -> unit) -> server
-[@@ocaml.deprecated
-" The signature and semantics of this function will soon change:
- - the callback parameter f will evaluate to a promise (-> unit Lwt.t),
- - channels will be closed automatically when that promise resolves, to avoid
-   leaking file descriptors, and
- - the result will be a promise (-> server Lwt.t).
- This will be breaking change in Lwt 3.0.0. See
-   https://github.com/ocsigen/lwt/pull/258
- To keep the current functionality, use Lwt_io.Versioned.establish_server_1
- To use the safer version immediately, use Lwt_io.Versioned.establish_server_2
- Both alternatives require Lwt >= 2.7.0."]
-  (** [establish_server ?fd ?buffer_size ?backlog sockaddr f] creates a server
-      which listens for incoming connections. New connections are passed to [f].
+val establish_server_with_client_socket :
+  ?server_fd:Lwt_unix.file_descr ->
+  ?backlog:int ->
+  ?no_close:bool ->
+  Unix.sockaddr ->
+  (Lwt_unix.sockaddr -> Lwt_unix.file_descr -> unit Lwt.t) ->
+    server Lwt.t
+(** [establish_server_with_client_socket listen_address f] creates a server
+    which listens for incoming connections on [listen_address]. When a client
+    makes a new connection, it is passed to [f]: more precisely, the server
+    calls
 
-      [establish_server] does not start separate threads for running [f], nor
-      close the connections passed to [f]. Thus, the skeleton of a practical
-      server based on [establish_server] might look like this:
+{[
+f client_address client_socket
+]}
 
-      {[
-        Lwt_io.establish_server address (fun (ic, oc) ->
-          Lwt.async (fun () ->
+    where [client_address] is the address (peer name) of the new client, and
+    [client_socket] is the socket connected to the client.
 
-            (* ... *)
+    The server does not block waiting for [f] to complete: it concurrently tries
+    to accept more client connections while [f] is handling the client.
 
-            Lwt.catch (fun () -> Lwt_io.close oc) (fun _ -> Lwt.return_unit) >>=
-            Lwt.catch (fun () -> Lwt_io.close ic) (fun _ -> Lwt.return_unit)))
-      ]}
+    When the promise returned by [f] completes (i.e., [f] is done handling the
+    client), [establish_server_with_client_socket] automatically closes
+    [client_socket]. This is a default behavior that is useful for simple cases,
+    but for a robust application you should explicitly close these channels
+    yourself, and handle any exceptions as appropriate. If the channels are
+    still open when [f] completes, and their automatic closing raises an
+    exception, [establish_server_with_client_address] treats it as an unhandled
+    exception reaching the top level of the application: it passes that
+    exception to {!Lwt.async_exception_hook}, the default behavior of which is
+    to print the exception and {e terminate your process}.
 
-      If [fd] is not specified, a fresh file descriptor will be created for
-      listening.
+    Automatic closing can be completely disabled by passing [~no_close:true].
 
-      [backlog] is the argument passed to [Lwt_unix.listen].
+    Similarly, if [f] raises an exception (or the promise it returns fails with
+    an exception), [establish_server_with_client_address] can do nothing with
+    that exception, except pass it to {!Lwt.async_exception_hook}.
 
-      @deprecated Will be replaced by {!Versioned.establish_server_2}, which
-        closes the channels passed to [f] automatically when a thread returned
-        by [f] completes. *)
+    [~server_fd] can be specified to use an existing file descriptor for
+    listening. Otherwise, a fresh socket is created internally. In either case,
+    [establish_server_with_client_socket] will internally assign
+    [listen_address] to the server socket.
 
-val shutdown_server : server -> unit
-[@@ocaml.deprecated
-" This function will soon evaluate to a promise that resolves when the close
- system call completes. This will be a breaking change in Lwt 3.0.0. See
-   https://github.com/ocsigen/lwt/issues/259
- To keep the current signature, use Lwt_io.Versioned.shutdown_server_1
- To use the new version immediately, use Lwt_io.Versioned.shutdown_server_2
- Both alternatives require Lwt >= 2.7.0."]
-  (** Closes the given server's listening socket. This function does not wait
-      for the [close] operation to actually complete. It does not affect the
-      sockets of connections that have already been accepted, i.e. passed to [f]
-      by [establish_server].
+    [~backlog] is the argument passed to {!Lwt_unix.listen}. Its default value
+    is [SOMAXCONN], which varies by platform and socket kind.
 
-      @deprecated Will be replaced by {!Versioned.shutdown_server_2}, which
-        evaluates to a thread that waits for [close] to complete. *)
+    The returned promise (a [server Lwt.t]) resolves when the server has just
+    started listening on [listen_address]: right after the internal call to
+    [listen], and right before the first internal call to [accept].
+
+    @since 4.1.0 *)
+
+val establish_server_with_client_address :
+  ?fd:Lwt_unix.file_descr ->
+  ?buffer_size:int ->
+  ?backlog:int ->
+  ?no_close:bool ->
+  Unix.sockaddr ->
+  (Lwt_unix.sockaddr -> input_channel * output_channel -> unit Lwt.t) ->
+    server Lwt.t
+(** Like {!Lwt_io.establish_server_with_client_socket}, but passes two buffered
+    channels to the connection handler [f]. These channels wrap the client
+    socket.
+
+    The channels are closed automatically when the promise returned by [f]
+    resolves. To avoid this behavior, pass [~no_close:true].
+
+    @since 3.1.0 *)
+
+val shutdown_server : server -> unit Lwt.t
+(** Closes the given server's listening socket. The returned promise resolves
+    when the [close(2)] system call completes. This function does not affect the
+    sockets of connections that have already been accepted, i.e. passed to [f]
+    by {!establish_server}.
+
+    @since 3.0.0 *)
 
 val lines_of_file : file_name -> string Lwt_stream.t
   (** [lines_of_file name] returns a stream of all lines of the file
@@ -545,8 +725,7 @@ val system_byte_order : byte_order
 val block : 'a channel  -> int -> (Lwt_bytes.t -> int -> 'b Lwt.t) -> 'b Lwt.t
   (** [block ch size f] pass to [f] the internal buffer and an
       offset. The buffer contains [size] chars at [offset]. [f] may
-      read or write these chars.  [size] must satisfy [0 <= size <=
-      16] *)
+      read or write these chars.  [size] must satisfy [0 <= size <= 16] *)
 
 (** Information for directly accessing the internal buffer of a
     channel *)
@@ -563,7 +742,8 @@ type direct_access = {
   (** - for input channels:
         refills the buffer and returns how many bytes have been read
       - for output channels:
-        flush partially the buffer and returns how many bytes have been written *)
+        flush partially the buffer and returns how many bytes have been
+        written *)
 }
 
 val direct_access : 'a channel -> (direct_access -> 'b Lwt.t) -> 'b Lwt.t
@@ -583,6 +763,23 @@ val set_default_buffer_size : int -> unit
       @raise Invalid_argument if the given size is smaller than [16]
       or greater than [Sys.max_string_length] *)
 
+(** {2 Deprecated} *)
+
+val establish_server :
+  ?fd : Lwt_unix.file_descr ->
+  ?buffer_size : int ->
+  ?backlog : int ->
+  ?no_close : bool ->
+  Unix.sockaddr -> (input_channel * output_channel -> unit Lwt.t) ->
+    server Lwt.t
+  [@@ocaml.deprecated
+"  Since Lwt 3.1.0, use Lwt_io.establish_server_with_client_address"]
+(** Like [establish_server_with_client_address], but does not pass the client
+    address or fd to the callback [f].
+
+    @deprecated Use {!establish_server_with_client_address}.
+    @since 3.0.0 *)
+
 (** Versioned variants of APIs undergoing breaking changes. *)
 module Versioned :
 sig
@@ -590,13 +787,17 @@ sig
     ?fd : Lwt_unix.file_descr ->
     ?buffer_size : int ->
     ?backlog : int ->
-    Unix.sockaddr -> (input_channel * output_channel -> unit) -> server
-  [@@ocaml.deprecated
-" Deprecated in favor of Lwt_io.Versioned.establish_server_2. See
+    Unix.sockaddr -> (input_channel * output_channel -> unit) ->
+      server
+    [@@ocaml.deprecated
+" Deprecated in favor of Lwt_io.establish_server. See
    https://github.com/ocsigen/lwt/pull/258"]
-  (** Alias for the current {!Lwt_io.establish_server}.
+  (** Old version of {!Lwt_io.establish_server}. The current
+      {!Lwt_io.establish_server} automatically closes channels passed to the
+      callback, and notifies the caller when the server's listening socket is
+      bound.
 
-      @deprecated Use {!establish_server_2}.
+      @deprecated Use {!Lwt_io.establish_server_with_client_address}.
       @since 2.7.0 *)
 
   val establish_server_2 :
@@ -606,43 +807,29 @@ sig
     ?no_close : bool ->
     Unix.sockaddr -> (input_channel * output_channel -> unit Lwt.t) ->
       server Lwt.t
-  (** [establish_server_2 sockaddr f] creates a server which listens for
-      incoming connections. New connections are passed to [f]. When threads
-      returned by [f] complete, the connections are closed automatically. To
-      prevent automatic closing, apply [establish_server_2] with
-      [~no_close:true].
-
-      The [?fd] and [?backlog] arguments have the same meaning as in
-      {!Lwt_io.establish_server}. [?buffer_size] sets the internal buffer size
-      of the channels passed to [f].
-
-      The server does not wait for each thread. It begins accepting new
-      connections immediately.
-
-      If a thread raises an exception, it is passed to
-      [!Lwt.async_exception_hook]. Likewise, if the automatic [close] of a
-      connection raises an exception, it is passed to
-      [!Lwt.async_exception_hook]. To robustly handle these exceptions, you
-      should call [close] manually inside [f], and use your own handler.
+    [@@ocaml.deprecated
+" In Lwt >= 3.0.0, this is an alias for Lwt_io.establish_server."]
+  (** Since Lwt 3.0.0, this is just an alias for {!Lwt_io.establish_server}.
 
+      @deprecated Use {!Lwt_io.establish_server_with_client_address}.
       @since 2.7.0 *)
 
   val shutdown_server_1 : server -> unit
-  [@@ocaml.deprecated
-" Deprecated in favor of Lwt_io.Versioned.shutdown_server_2. See
+    [@@ocaml.deprecated
+" Deprecated in favor of Lwt_io.shutdown_server. See
    https://github.com/ocsigen/lwt/issues/259"]
-  (** Alias for the current {!Lwt_io.shutdown_server}.
+  (** Old version of {!Lwt_io.shutdown_server}. The current
+      {!Lwt_io.shutdown_server} returns a promise, which resolves when the
+      server's listening socket is closed.
 
-      @deprecated Use {!shutdown_server_2}.
+      @deprecated Use {!Lwt_io.shutdown_server}.
       @since 2.7.0 *)
 
   val shutdown_server_2 : server -> unit Lwt.t
-  (** Closes the given server's listening socket. The thread returned by this
-      function waits for the underlying [close] system call to complete.
-
-      This function does not affect sockets of connections that have already
-      been accepted by the server, i.e. those passed by [establish_server] to
-      its callback [f].
+    [@@ocaml.deprecated
+" In Lwt >= 3.0.0, this is an alias for Lwt_io.shutdown_server."]
+  (** Since Lwt 3.0.0, this is just an alias for {!Lwt_io.shutdown_server}.
 
+      @deprecated Use {!Lwt_io.shutdown_server}.
       @since 2.7.0 *)
 end
