@@ -1,26 +1,7 @@
-(* Lightweight thread library for OCaml
- * http://www.ocsigen.org/lwt
- * Interface Lwt_unix
- * Copyright (C) 2005-2008 Jérôme Vouillon
- * Laboratoire PPS - CNRS Université Paris Diderot
- *                    2009 Jérémie Dimino
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU Lesser General Public License as
- * published by the Free Software Foundation, with linking exceptions;
- * either version 2.1 of the License, or (at your option) any later
- * version. See COPYING file for details.
- *
- * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
- *)
+(* This file is part of Lwt, released under the MIT license. See LICENSE.md for
+   details, or visit https://github.com/ocsigen/lwt/blob/master/LICENSE.md. *)
+
+
 
 (** Cooperative system calls *)
 
@@ -64,70 +45,6 @@ val handle_unix_error : ('a -> 'b Lwt.t) -> 'a -> 'b Lwt.t
   (** Same as [Unix.handle_unix_error] but catches lwt-level
       exceptions *)
 
-(** {2 Configuration} *)
-
-(** For system calls that cannot be made asynchronously, Lwt uses one
-    of the following method: *)
-type async_method =
-  | Async_none
-      (** System calls are made synchronously, and may block the
-          entire program. *)
-  | Async_detach
-      (** System calls are made in another system thread, thus without
-          blocking other Lwt threads. The drawback is that it may
-          degrade performance in some cases.
-
-          This is the default. *)
-  | Async_switch
-      (** System calls are made in the main thread, and if one blocks
-          the execution continue in another system thread. This method
-          is the most efficient, also you will get better performance
-          if you force all threads to run on the same cpu. On linux
-          this can be done by using the command [taskset].
-
-          Note that this method is still experimental. *)
-
-val default_async_method : unit -> async_method
-  (** Returns the default async method.
-
-      This can be initialized using the environment variable
-      ["LWT_ASYNC_METHOD"] with possible values ["none"],
-      ["detach"] and ["switch"]. *)
-
-val set_default_async_method : async_method -> unit
-  (** Sets the default async method. *)
-
-val async_method : unit -> async_method
-  (** [async_method ()] returns the async method used in the current
-      thread. *)
-
-val async_method_key : async_method Lwt.key
-  (** The key for storing the local async method. *)
-
-val with_async_none : (unit -> 'a) -> 'a
-  (** [with_async_none f] is a shorthand for:
-
-      {[
-        Lwt.with_value async_method_key (Some Async_none) f
-      ]}
-  *)
-
-val with_async_detach : (unit -> 'a) -> 'a
-  (** [with_async_detach f] is a shorthand for:
-
-      {[
-        Lwt.with_value async_method_key (Some Async_detach) f
-      ]}
-  *)
-
-val with_async_switch : (unit -> 'a) -> 'a
-  (** [with_async_switch f] is a shorthand for:
-
-      {[
-        Lwt.with_value async_method_key (Some Async_switch) f
-      ]}
-  *)
-
 (** {2 Sleeping} *)
 
 val sleep : float -> unit Lwt.t
@@ -139,8 +56,13 @@ val yield : unit -> unit Lwt.t
       as soon as possible and terminates. *)
 
 val auto_yield : float -> (unit -> unit Lwt.t)
-  (** [auto_yield timeout] returns a function [f] that will yield
-      every [timeout] seconds. *)
+  (** [auto_yield timeout] returns a function [f], and [f ()] has the following
+      behavior:
+
+      - If it has been more than [timeout] seconds since the last time [f ()]
+        behaved like {!Lwt_unix.yield}, [f ()] calls {!Lwt_unix.yield}.
+      - Otherwise, if it has been less than [timeout] seconds, [f ()] behaves
+        like {!Lwt.return_unit}, i.e. it does not yield. *)
 
 exception Timeout
   (** Exception raised by timeout operations *)
@@ -187,7 +109,7 @@ val state : file_descr -> state
 val unix_file_descr : file_descr -> Unix.file_descr
   (** Returns the underlying unix {b file descriptor}. It always
       succeeds, even if the {b file descriptor}'s state is not
-      {!Open}. *)
+      [Open]. *)
 
 val of_unix_file_descr : ?blocking : bool -> ?set_flags : bool -> Unix.file_descr -> file_descr
 (** Wraps a [Unix] file descriptor [fd] in an [Lwt_unix.file_descr] [fd'].
@@ -201,7 +123,10 @@ val of_unix_file_descr : ?blocking : bool -> ?set_flags : bool -> Unix.file_desc
 
     If [~blocking] is not specified, [of_unix_file_descr] chooses non-blocking
     mode for Unix sockets, Unix pipes, and Windows sockets, and blocking mode
-    for everything else.
+    for everything else. {b Note:} not specifying [~blocking] causes [fstat] to
+    be lazily called on [fd], the first time your code performs I/O on [fd'].
+    This [fstat] call can be expensive, so if you use [of_unix_file_descr] a
+    lot, be sure to specify [~blocking] explicitly.
 
     [of_unix_file_descr] runs a system call to set the specified or chosen
     blocking mode on the underlying [fd].
@@ -244,7 +169,7 @@ val set_blocking : ?set_flags : bool -> file_descr -> bool -> unit
 val abort : file_descr -> exn -> unit
   (** [abort fd exn] makes all current and further uses of the file
       descriptor fail with the given exception. This put the {b file
-      descriptor} into the {!Aborted} state.
+      descriptor} into the [Aborted] state.
 
       If the {b file descriptor} is closed, this does nothing, if it is
       aborted, this replace the abort exception by [exn].
@@ -263,7 +188,10 @@ val fork : unit -> int
       - in the child process all pending jobs are canceled,
       - if you are going to use Lwt in the parent and the child, it is
         a good idea to call {!Lwt_io.flush_all} before callling
-        {!fork} to avoid double-flush. *)
+        {!fork} to avoid double-flush.
+      - otherwise, if you will not use Lwt in the child, call
+        {!Lwt_main.Exit_hooks.remove_all} to avoid Lwt calling {!Lwt_main.run}
+        during process exit. *)
 
 type process_status =
     Unix.process_status =
@@ -280,7 +208,10 @@ val wait : unit -> (int * process_status) Lwt.t
   (** Wrapper for [Unix.wait] *)
 
 val waitpid : wait_flag list -> int -> (int * process_status) Lwt.t
-  (** Wrapper for [Unix.waitpid] *)
+(** A promise-returning analog to
+    {{: https://caml.inria.fr/pub/docs/manual-ocaml/libref/Unix.html#VALwaitpid}
+    [Unix.waitpid]}. This call is non-blocking on Unix-like systems, but is
+    always blocking on Windows. *)
 
 (** Resource usages *)
 type resource_usage = {
@@ -292,8 +223,8 @@ type resource_usage = {
 }
 
 val wait4 : wait_flag list -> int -> (int * process_status * resource_usage) Lwt.t
-  (** [wait4 flags pid] returns [(pid, status, rusage)] where [(pid,
-      status)] is the same result as [Unix.waitpid flags pid], and
+  (** [wait4 flags pid] returns [(pid, status, rusage)] where [(pid, status)]
+      is the same result as [Unix.waitpid flags pid], and
       [rusage] contains accounting information about the child.
 
       On windows it will always returns [{ utime = 0.0; stime = 0.0 }]. *)
@@ -347,7 +278,7 @@ val openfile : string -> open_flag list -> file_perm -> file_descr Lwt.t
 
 val close : file_descr -> unit Lwt.t
   (** Close a {b file descriptor}. This close the underlying unix {b
-      file descriptor} and set its state to {!Closed} *)
+      file descriptor} and set its state to [Closed]. *)
 
 val read : file_descr -> bytes -> int -> int -> int Lwt.t
 (** [read fd buf ofs len] reads up to [len] bytes from [fd], and writes them to
@@ -422,6 +353,9 @@ sig
   (** [is_empty vs] is [true] if and only if [vs] has no I/O vectors, or all I/O
       vectors in [vs] have zero bytes. *)
 
+  val byte_count : t -> int
+  (** [byte_count vs] is the total number of bytes in [vs]. *)
+
   val system_limit : int option
   (** Some systems limit the number of I/O vectors that can be passed in a
       single call to their [writev] or [readv] system calls. On those systems,
@@ -498,12 +432,27 @@ val writable : file_descr -> bool
       writable. *)
 
 val wait_read : file_descr -> unit Lwt.t
-  (** waits (without blocking other threads) until there is something
-      to read on the file descriptor *)
+  (** Waits (without blocking other threads) until there is something
+      to read from the file descriptor.
+
+      Note that you don't need to use this function if you are
+      using Lwt I/O functions for reading, since they provide
+      non-blocking waiting automatically.
+
+      The intended use case for this function is interfacing with
+      existing libraries that are known to be blocking. *)
 
 val wait_write : file_descr -> unit Lwt.t
-  (** waits (without blocking other threads) until it is possible to
-      write on the file descriptor *)
+  (** Waits (without blocking other threads) until it is possible to
+      write on the file descriptor.
+
+      Note that you don't need to use this function if you are
+      using Lwt I/O functions for writing, since they provide
+      non-blocking waiting automatically.
+
+      The intended use case for this function is interfacing with
+      existing libraries that are known to be blocking. *)
+
 
 (** {2 Seeking and truncating} *)
 
@@ -716,6 +665,9 @@ val rmdir : string -> unit Lwt.t
 val chdir : string -> unit Lwt.t
   (** Wrapper for [Unix.chdir] *)
 
+val getcwd : unit -> string Lwt.t
+  (** Wrapper for [Unix.getcwd] *)
+
 val chroot : string -> unit Lwt.t
   (** Wrapper for [Unix.chroot] *)
 
@@ -859,7 +811,7 @@ val signal_count : unit -> int
 val reinstall_signal_handler : int -> unit
   (** [reinstall_signal_handler signum] if any signal handler is
       registered for this signal with {!on_signal}, it reinstall the
-      signal handler (with [Sys.set_signal]). This is usefull in case
+      signal handler (with [Sys.set_signal]). This is useful in case
       another part of the program install another signal handler. *)
 
 (** {2 Sockets} *)
@@ -888,24 +840,13 @@ val socket : socket_domain -> socket_type -> int -> file_descr
 val socketpair : socket_domain -> socket_type -> int -> file_descr * file_descr
   (** Wrapper for [Unix.socketpair] *)
 
-val bind : file_descr -> sockaddr -> unit
-  [@@ocaml.deprecated
-" This function will soon evaluate to a promise (-> unit Lwt.t), because the
- bind system call can block for Unix domain sockets. See
-   https://github.com/ocsigen/lwt/issues/230
- This will be a breaking change in Lwt 3.0.0.
- If you don't use Unix domain sockets and use Lwt_unix.bind ... ; rather than
- let () = Lwt_unix.bind ... in, you can ignore this warning.
- To retain the current signature, use Lwt_unix.Versioned.bind_1
- To use the new non-blocking version immediately, use Lwt_unix.Versioned.bind_2
- Both alternatives require Lwt >= 2.7.0."]
+val bind : file_descr -> sockaddr -> unit Lwt.t
 (** Binds an address to the given socket. This is the cooperative analog of
     {{:http://caml.inria.fr/pub/docs/manual-ocaml/libref/Unix.html#VALbind}
     [Unix.bind]}. See also
     {{:http://man7.org/linux/man-pages/man3/bind.3p.html} [bind(3p)]}.
 
-    @deprecated Will be replaced by {!Versioned.bind_2}, whose result type is
-      [unit Lwt.t] instead of [unit]. *)
+    @since 3.0.0 *)
 
 val listen : file_descr -> int -> unit
   (** Wrapper for [Unix.listen] *)
@@ -979,17 +920,9 @@ val sendto : file_descr -> bytes -> int -> int -> msg_flag list -> sockaddr -> i
 
     On Windows, [sendto] copies the given buffer before writing. *)
 
-(** An io-vector. Used by {!recv_msg} and {!send_msg}. *)
-type io_vector = {
-  iov_buffer : string;
-  iov_offset : int;
-  iov_length : int;
-}
-
-val io_vector : buffer : string -> offset : int -> length : int -> io_vector
-  (** Creates an io-vector *)
-
-val recv_msg : socket : file_descr -> io_vectors : io_vector list -> (int * Unix.file_descr list) Lwt.t
+val recv_msg :
+  socket:file_descr -> io_vectors:IO_vectors.t ->
+    (int * Unix.file_descr list) Lwt.t
 (** [recv_msg ~socket ~io_vectors] receives data into a list of
     io-vectors, plus any file-descriptors that may accompany the
     messages. It returns a tuple whose first field is the number of
@@ -998,17 +931,23 @@ val recv_msg : socket : file_descr -> io_vectors : io_vector list -> (int * Unix
     provided [io_vectors] list. Data is written directly into the
     [iov_buffer] buffers.
 
-    Not implemented on Windows. *)
+    Not implemented on Windows.
 
-val send_msg : socket : file_descr -> io_vectors : io_vector list -> fds : Unix.file_descr list -> int Lwt.t
+    @since 5.0.0 *)
+
+val send_msg :
+  socket:file_descr -> io_vectors:IO_vectors.t -> fds:Unix.file_descr list ->
+    int Lwt.t
 (** [send_msg ~socket ~io_vectors ~fds] sends data from a list of
     io-vectors, accompanied with a list of file-descriptors. It
     returns the number of bytes sent. If fd-passing is not possible on
     the current system and [fds] is not empty, it raises
     [Lwt_sys.Not_available "fd_passing"]. Data is written directly from
-    the [iov_buffer] buffers.
+    the [io_vectors] buffers.
 
-    Not implemented on Windows. *)
+    Not implemented on Windows.
+
+    @since 5.0.0 *)
 
 type credentials = {
   cred_pid : int;
@@ -1269,6 +1208,102 @@ type flow_action =
 val tcflow : file_descr -> flow_action -> unit Lwt.t
   (** Wrapper for [Unix.tcflow] *)
 
+
+
+(** {2 Configuration (deprecated)} *)
+
+(** For system calls that cannot be made asynchronously, Lwt uses one
+    of the following method: *)
+type async_method =
+  | Async_none
+      (** System calls are made synchronously, and may block the
+          entire program. *)
+  | Async_detach
+      (** System calls are made in another system thread, thus without
+          blocking other Lwt threads. The drawback is that it may
+          degrade performance in some cases.
+
+          This is the default. *)
+  | Async_switch
+    [@ocaml.deprecated " Use Lwt_unix.Async_detach."]
+      (** Currently a synonym for [Async_detach]. This was a different method in
+          the past. *)
+
+val default_async_method : unit -> async_method
+  [@@ocaml.deprecated
+" Will always return Async_detach in Lwt >= 5.0.0. See
+   https://github.com/ocsigen/lwt/issues/572"]
+(** Returns the default async method.
+
+    This can be initialized using the environment variable
+    ["LWT_ASYNC_METHOD"] with possible values ["none"],
+    ["detach"] and ["switch"].
+
+    @deprecated Will always return [Async_detach] in Lwt 5.0.0. *)
+
+val set_default_async_method : async_method -> unit
+  [@@ocaml.deprecated
+" Will be a no-op in Lwt >= 5.0.0. See
+   https://github.com/ocsigen/lwt/issues/572"]
+(** Sets the default async method.
+
+    @deprecated Will be a no-op in Lwt 5.0.0. *)
+
+val async_method : unit -> async_method
+  [@@ocaml.deprecated
+" Will always return Async_detach in Lwt >= 5.0.0. See
+   https://github.com/ocsigen/lwt/issues/572"]
+(** [async_method ()] returns the async method used in the current
+    thread.
+
+    @deprecated Will always return [Async_detach] in Lwt 5.0.0. *)
+
+val async_method_key : async_method Lwt.key
+  [@@ocaml.deprecated
+" Will be ignored in Lwt >= 5.0.0. See
+   https://github.com/ocsigen/lwt/issues/572"]
+(** The key for storing the local async method.
+
+    @deprecated Will be ignored in Lwt 5.0.0. *)
+
+val with_async_none : (unit -> 'a) -> 'a
+  [@@ocaml.deprecated
+" Will have no effect in Lwt >= 5.0.0. See
+   https://github.com/ocsigen/lwt/issues/572"]
+(** [with_async_none f] is a shorthand for:
+
+    {[
+      Lwt.with_value async_method_key (Some Async_none) f
+    ]}
+
+    @deprecated Will have no effect in Lwt 5.0.0. *)
+
+val with_async_detach : (unit -> 'a) -> 'a
+  [@@ocaml.deprecated
+" Will have no effect in Lwt >= 5.0.0. See
+   https://github.com/ocsigen/lwt/issues/572"]
+(** [with_async_detach f] is a shorthand for:
+
+    {[
+      Lwt.with_value async_method_key (Some Async_detach) f
+    ]}
+
+    @deprecated Will have no effect in Lwt 5.0.0. *)
+
+val with_async_switch : (unit -> 'a) -> 'a
+  [@@ocaml.deprecated
+" Will have no effect in Lwt >= 5.0.0. See
+   https://github.com/ocsigen/lwt/issues/572"]
+(** [with_async_switch f] is a shorthand for:
+
+    {[
+      Lwt.with_value async_method_key (Some Async_switch) f
+    ]}
+
+    @deprecated Will have no effect in Lwt 5.0.0. *)
+
+
+
 (** {2 Low-level interaction} *)
 
 exception Retry
@@ -1296,7 +1331,7 @@ val wrap_syscall : io_event -> file_descr -> (unit -> 'a) -> 'a Lwt.t
 
 val check_descriptor : file_descr -> unit
   (** [check_descriptor fd] raise an exception if [fd] is not in the
-      state {!Open} *)
+      state [Open]. *)
 
 val register_action : io_event -> file_descr -> (unit -> 'a) -> 'a Lwt.t
   (** [register_action set fd action] registers [action] on [fd]. When
@@ -1319,7 +1354,10 @@ val run_job : ?async_method : async_method -> 'a job -> 'a Lwt.t
   (** [run_job ?async_method job] starts [job] and wait for its
       termination.
 
-      The async method is choosen follow:
+      The [~async_method] argument will be ignored in Lwt 5.0.0, and this
+      function will always act as if [~async_method:Async_detach] is passed.
+
+      The async method is chosen follow:
       - if the optional parameter [async_method] is specified, it is
         used,
       - otherwise if the local key {!async_method_key} is set in the
@@ -1327,15 +1365,15 @@ val run_job : ?async_method : async_method -> 'a job -> 'a Lwt.t
       - otherwise the default method (returned by
         {!default_async_method}) is used.
 
-      If the method is {!Async_none} then the job is run synchronously
+      If the method is [Async_none] then the job is run synchronously
       and may block the current system thread, thus blocking all Lwt
       threads.
 
-      If the method is {!Async_detach} then the job is run in another
+      If the method is [Async_detach] then the job is run in another
       system thread, unless the the maximum number of worker threads
       has been reached (as given by {!pool_size}).
 
-      If the method is {!Async_switch} then the job is run
+      If the method is [Async_switch] then the job is run
       synchronously and if it blocks, execution will continue in
       another system thread (unless the limit is reached).
   *)
@@ -1393,8 +1431,8 @@ val set_notification : int -> (unit -> unit) -> unit
 
 (** {2 System threads pool} *)
 
-(** If the program is using the async method {!Async_detach} or
-    {!Async_switch}, Lwt will launch system threads to execute
+(** If the program is using the async method [Async_detach] or
+    [Async_switch], Lwt will launch system threads to execute
     blocking system calls asynchronously. *)
 
 val pool_size : unit -> int
@@ -1432,18 +1470,42 @@ module Versioned :
 sig
   val bind_1 : file_descr -> sockaddr -> unit
     [@@ocaml.deprecated
-" Deprecated in favor of Lwt_unix.Versioned.bind_2. See
+" Deprecated in favor of Lwt_unix.bind. See
    https://github.com/ocsigen/lwt/issues/230"]
-  (** Alias for the current {!Lwt_unix.bind}.
+  (** Old version of {!Lwt_unix.bind}. The current {!Lwt_unix.bind} evaluates to
+      a promise, because the internal [bind(2)] system call can block if the
+      given socket is a Unix domain socket.
 
-      @deprecated Use {!bind_2}.
+      @deprecated Use {!Lwt_unix.bind}.
       @since 2.7.0 *)
 
   val bind_2 : file_descr -> sockaddr -> unit Lwt.t
-  (** Like {!Lwt_unix.bind}, but evaluates to an Lwt thread, in order to avoid
-      blocking the process in case the given socket is a Unix domain socket.
+    [@@ocaml.deprecated
+" In Lwt >= 3.0.0, this is an alias for Lwt_unix.bind."]
+  (** Since Lwt 3.0.0, this is just an alias for {!Lwt_unix.bind}.
 
+      @deprecated Use {!Lwt_unix.bind}.
       @since 2.7.0 *)
+
+  val recv_msg_2 :
+    socket:file_descr -> io_vectors:IO_vectors.t ->
+      (int * Unix.file_descr list) Lwt.t
+    [@@ocaml.deprecated
+" In Lwt >= 5.0.0, this is an alias for Lwt_unix.recv_msg."]
+  (** Since Lwt 5.0.0, this is an alias for {!Lwt_unix.recv_msg}.
+
+      @deprecated Use {!Lwt_unix.recv_msg}.
+      @since 4.3.0 *)
+
+  val send_msg_2 :
+    socket:file_descr -> io_vectors:IO_vectors.t -> fds:Unix.file_descr list ->
+      int Lwt.t
+    [@@ocaml.deprecated
+" In Lwt >= 5.0.0, this is an alias for Lwt_unix.send_msg."]
+  (** Since Lwt 5.0.0, this is an alias for {!Lwt_unix.send_msg}.
+
+      @deprecated Use {!Lwt_unix.send_msg}.
+      @since 4.3.0 *)
 end
 
 (**/**)
@@ -1455,3 +1517,17 @@ val run : 'a Lwt.t -> 'a
 val has_wait4 : bool
   [@@ocaml.deprecated " Use Lwt_sys.have `wait4."]
   (** @deprecated Use [Lwt_sys.have `wait4]. *)
+
+val somaxconn : unit -> int
+  [@@ocaml.deprecated " This is an internal function."]
+
+val retained : 'a -> bool ref
+  (** @deprecated Used for testing. *)
+
+val read_bigarray :
+  string -> file_descr -> IO_vectors._bigarray -> int -> int -> int Lwt.t
+  [@@ocaml.deprecated " This is an internal function."]
+
+val write_bigarray :
+  string -> file_descr -> IO_vectors._bigarray -> int -> int -> int Lwt.t
+  [@@ocaml.deprecated " This is an internal function."]
